/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.sensor.fritzmeier;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.bean.ColumnPositionMappingStrategy;
import au.com.bytecode.opencsv.bean.CsvToBean;
import com.vividsolutions.jts.geom.Coordinate;
import eu.agrosense.client.grid.Grid;
import eu.agrosense.client.grid.GridProvider;
import java.beans.PropertyEditor;
import java.beans.PropertyEditorManager;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.Icon;
import nl.cloudfarming.client.imp.ImportProcessor;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.model.SensorObservation;
import nl.cloudfarming.client.sensor.AbstractSensorImportProcessor;
import nl.cloudfarming.client.sensor.SensorDataObject;
import nl.cloudfarming.client.sensor.SensorPalette;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages("sensor fritzmeier format description=FritzMeier Sensor file (*.csv)")
@ServiceProvider(service = ImportProcessor.class)
public class FritzMeierImportProcessor extends AbstractSensorImportProcessor {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(SensorDataObject.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(SensorDataObject.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(SensorDataObject.class, "sensor.common.fileprocessor.loginfo.process_start");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(SensorDataObject.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(FritzMeierImportProcessor.class, "sensor.fritzmeier.fileprocessor.log.displayname");
    // Other variable declarations
    private final String delimiter = FritzmeierFileDescription.getDelimiter();
    private final ProgressHandle progressHandle;
    PropertyEditor old;
    private static final String[] COLUMS = new String[]{"latitude", "longitude", "ddateStr", "utcTimeStr", "course", "latitudeOriginal", "longitudeOriginal", "altitude", "speed", "fixType", "irmi", "tcari", "osavi", "tcOs", "wdvi", "ndre"};
    private CsvToBean csv;
    private ColumnPositionMappingStrategy strat;
    private static final AppLogger LOG = AppLogFactory.getLogger(FritzMeierImportProcessor.class);
    public static final String FRITZMEIER_FOLDER = "Fritzmeier";
    private static final Icon ICON = ImageUtilities.loadImageIcon(FritzmeierDataObject.ICON_BASE, true);

    public FritzMeierImportProcessor() {
        // assert dataObject != null;
        LOG.entering("FritzmeierFileProcessor", "constructor");
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(SensorDataObject.class, "sensor.common.fileprocessor.loginfo.process_description", getFormatDescription()));
        old = PropertyEditorManager.findEditor(Date.class);
        strat = new ColumnPositionMappingStrategy();
        strat.setType(FritzmeierSensorData.class);
        strat.setColumnMapping(COLUMS);
        csv = new CsvToBean();
    }

    @Override
    public String[] getSupportedExtentions() {
        return new String[]{"csv"};
    }

    @Override
    public boolean isSupported(FileObject importFile) {
        return FritzmeierDataObject.MIME_TYPE.equals(importFile.getMIMEType());
    }

    @Override
    public String getFormatDescription() {
        return Bundle.sensor_fritzmeier_format_description();
    }

    @Override
    public Icon getIcon() {
        return ICON;
    }

    @Override
    public void process(FileObject importFile, Project project) throws IllegalArgumentException {
        process(importFile, project, new ArrayList());
    }

    @Override
    public void process(FileObject importFile, Project project, List toPopulate) throws IllegalArgumentException {
        List<SensorObservation> sensorData = new ArrayList<SensorObservation>();

        try {
            int success = 0;
            int progress = 0;
            int fail = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), importFile.getName());

            try {
                this.progressHandle.start(Long.valueOf(importFile.asLines().size()).intValue());

                for (String line : importFile.asLines()) {
                    progressHandle.progress(++progress);
                    if (!isHeader(line)) {
                        try {
                            sensorData.add(parse(line));
                            success++;
                        } catch (Exception ex) {
                            fail++;
                        }
                    }
                }
                LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);
                if (fail > 0) {
                    LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
                }
            } finally {
                progressHandle.finish();
            }
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, importFile.getName());
            LOG.trace(ex);
        }
        if (!sensorData.isEmpty()) {


            double minx = sensorData.get(0).getLongitude();
            double maxx = sensorData.get(0).getLongitude();
            double miny = sensorData.get(0).getLatitude();
            double maxy = sensorData.get(0).getLatitude();

            for (SensorObservation sd : sensorData) {
                minx = (minx < sd.getLongitude()) ? minx : sd.getLongitude();
                miny = (miny < sd.getLatitude()) ? miny : sd.getLatitude();
                maxx = (maxx > sd.getLongitude()) ? maxx : sd.getLongitude();
                maxy = (maxy > sd.getLatitude()) ? maxy : sd.getLatitude();
            }

            GridProvider gridProvider = Lookup.getDefault().lookup(GridProvider.class);
            Grid grid = gridProvider.getDefaultGridSection(project.getLookup().lookup(EnterpriseIdProvider.class),new Coordinate(minx, miny), new Coordinate(maxx, maxy));

            for (SensorObservation sd : sensorData) {
                //TODO sample data gets casted into an int, so multiply with 1000 for example
                if (sd.getNdvi() != null) {
                    grid.setRasterValue(new Coordinate(sd.getLongitude(), sd.getLatitude()), (int) (sd.getNdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
                }

                if (sd.getWdvi() != null) {
                    grid.setRasterValue(new Coordinate(sd.getLongitude(), sd.getLatitude()), (int) (sd.getWdvi() * SensorPalette.SENSOR_DATA_MULTIPLIER));
                }
            }
            toPopulate.add(grid);
        }
    }

    @Override
    protected FileObject getSensorImportSubFolder(Project project) throws IOException {
        return getSubFolder(getSensorImportRootFolder(project), FRITZMEIER_FOLDER);
    }

    /**
     * Parses payload to FritzmeierSensorData
     *
     * @param payload Fritzmeier sensor line
     * @param dataFile
     * @return FritzmeierSensorData object
     * @throws ParseException
     * @throws FritzmeierParseException
     */
    public FritzmeierSensorData parse(String payload) throws FritzmeierParseException {
        try {
            // Use delimiter defined in FritzmeierFileDescription
            CSVReader csvReader = new CSVReader(new StringReader(payload), delimiter.charAt(0));
            List<FritzmeierSensorData> list = csv.parse(strat, csvReader);
            return (FritzmeierSensorData) list.get(0);
        } catch (RuntimeException ex) {
            LOG.error(ex.getMessage());
            throw new FritzmeierParseException(ex.getCause());
        }
    }

    public boolean isHeader(String line) {
        return line.startsWith("$");
    }
    

    /**
     * Method which will create a SensorMetadata object from the preferences:
     * FritzmeierPreference.SENSOR_DATA_FILE_FORMAT
     * FritzmeierPreference.CROPCIRCLE_SENSOR_HEIGTH
     * FritzmeierPreference.NUMBER_OF_CROPCIRCLE_SENSORS
     * FritzmeierPreference.CROPCIRCLE_SENSOR_SPREAD
     * FritzmeierPreference.SENSOR_TYPE FritzmeierPreference.SPECTRAL_CHANNELS
     *
     * @return created SensorMetadata object
     */
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = FritzmeierPreference.SENSOR_DATA_FILE_FORMAT.getValue();
        int sensorHeigth = Integer.parseInt(FritzmeierPreference.FRITZMEIER_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(FritzmeierPreference.NUMBER_OF_FRITZMEIER_SENSORS.getValue());
        String FritzmeierSensorSpread = FritzmeierPreference.FRITZMEIER_SENSOR_SPREAD.getValue();
        String sensorType = FritzmeierPreference.SENSOR_TYPE.getValue();
        String spectralChannels = FritzmeierPreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, FritzmeierSensorSpread, sensorType, spectralChannels);
    }

    protected String getFileDateFormat() {
        return FritzmeierPreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }    
}
